import { logger } from "@libp2p/logger";
import * as Comlink from "comlink";
import sqlite3InitModule from "@sqlite.org/sqlite-wasm";
import { AbstractModelDB, parseConfig, } from "@canvas-js/modeldb";
import { InnerModelDB } from "./InnerModelDB.js";
import "./worker.js";
export class ModelDB extends AbstractModelDB {
    static async initialize({ path, models }) {
        const config = parseConfig(models);
        let worker, wrappedDB;
        if (path) {
            worker = new Worker(new URL("./worker.js", import.meta.url), { type: "module" });
            const initializeDB = Comlink.wrap(worker);
            const logProxy = Comlink.proxy(logger("canvas:modeldb:worker"));
            wrappedDB = (await initializeDB(path, config, logProxy));
        }
        else {
            worker = null;
            const sqlite3 = await sqlite3InitModule({ print: console.log, printErr: console.error });
            const db = new sqlite3.oo1.DB();
            const log = logger("canvas:modeldb:transient");
            wrappedDB = new InnerModelDB(db, config, log);
        }
        return new ModelDB({ worker: worker, wrappedDB, config });
    }
    constructor({ worker, wrappedDB, config, }) {
        super(config);
        Object.defineProperty(this, "worker", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "wrappedDB", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.worker = worker;
        this.wrappedDB = wrappedDB;
    }
    async close() {
        this.log("closing");
        await this.wrappedDB.close();
        if (this.worker !== null) {
            this.worker.terminate();
        }
    }
    async apply(effects) {
        await this.wrappedDB.apply(effects);
        for (const { model, query, filter, callback } of this.subscriptions.values()) {
            if (effects.some(filter)) {
                try {
                    const queryRes = await this.query(model, query);
                    callback(queryRes);
                }
                catch (err) {
                    console.error(err);
                }
            }
        }
    }
    async get(modelName, key) {
        // @ts-ignore
        return this.wrappedDB.get(modelName, key);
    }
    async getMany(modelName, keys) {
        // @ts-ignore
        return this.wrappedDB.getMany(modelName, keys);
    }
    async *iterate(modelName) {
        return this.wrappedDB.iterate(modelName);
    }
    async count(modelName, where) {
        return this.wrappedDB.count(modelName, where);
    }
    async clear(modelName) {
        return this.wrappedDB.clear(modelName);
    }
    async query(modelName, query = {}) {
        // @ts-ignore
        return this.wrappedDB.query(modelName, query);
    }
}
